  
#ifndef _coordsys_cmpt_h_
#define _coordsys_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/images/coordsys_forward.h>

using namespace std;

namespace casac {
           


   class  coordsys  {
       public :


         ~coordsys();
         coordsys();

         
                                      casac::coordsys* newcoordsys( bool  direction=false,  bool  spectral=false,  const  std::vector<std::string>& stokes=std::vector<string>(1, ""),  int  linear=0,  bool  tabular=false);

         bool addcoordinate( bool  direction=false,  bool  spectral=false,  const  std::vector<std::string>& stokes=std::vector<string>(1, ""),  int  linear=0,  bool  tabular=false);

         std::vector<int> axesmap( bool  toworld=true);

         std::vector<std::string> axiscoordinatetypes( bool  world=true);

         string conversiontype( const string& type=string("direction"));

         std::vector<double>convert( const std::vector<double>& coordin,  const  std::vector<bool>&  absin=initialize_vector(1, (bool)true),  const string& dopplerin=string("radio"),  const  std::vector<std::string>& unitsin=std::vector<string>(1, ""),  const  std::vector<bool>&  absout=initialize_vector(1, (bool)true),  const string& dopplerout=string("radio"),  const  std::vector<std::string>& unitsout=std::vector<string>(1, ""),  const  std::vector<int>& shape=initialize_vector(1, (int)-1));

         record* convertdirection( const string& frame);

         variant* convertmany( const variant & coordin,  const  std::vector<bool>&  absin=initialize_vector(1, (bool)true),  const string& dopplerin=string("radio"),  const  std::vector<std::string>& unitsin=std::vector<string>(1, ""),  const  std::vector<bool>&  absout=initialize_vector(1, (bool)true),  const string& dopplerout=string("radio"),  const  std::vector<std::string>& unitsout=std::vector<string>(1, ""),  const  std::vector<int>& shape=initialize_vector(1, (int)-1));

         std::vector<std::string> coordinatetype( int  which=-1);

         
                                      casac::coordsys* copy();

         bool done();

         record* epoch();

         record* findaxis( bool  world=true,  int  axis=0);

         int findaxisbyname(const  string& axisname=string(""),  bool  allowfriendlyname=true);

         record* findcoordinate( const string& type=string("direction"),  int  which=0);

         std::vector<double>frequencytofrequency( const std::vector<double>& value,  const string& frequnit=string(""),  const variant & velocity = variant());

         std::vector<double>frequencytovelocity( const std::vector<double>& value,  const string& frequnit=string(""),  const string& doppler=string("radio"),  const string& velunit=string("km/s"));

         bool fromrecord( const record& record);

         record* increment( const string& format=string("n"),  const string& type=string(""));

         variant* lineartransform( const string& type);

         std::vector<std::string> names( const string& type=string(""));

         int naxes( bool  world=true);

         int ncoordinates();

         string observer();

         record* projection( const string& type=string(""));

         std::vector<std::string> referencecode( const string& type=string(""),  bool  list=false);

         record* referencepixel( const string& type=string(""));

         record* referencevalue( const string& format=string("n"),  const string& type=string(""));

         bool reorder( const  std::vector<int>& order);

         bool transpose( const  std::vector<int>& order);

         bool replace( const record& csys,  int  whichin,  int  whichout);

         record* restfrequency();

         bool setconversiontype( const string& direction=string(""),  const string& spectral=string(""));

         string getconversiontype( const string& type=string(""),  bool  showconversion=true);

         bool setdirection( const string& refcode=string(""),  const string& proj=string(""),  const std::vector<double>& projpar=initialize_vector(1, (double)-1),  const std::vector<double>& refpix=initialize_vector(1, (double)-1),  const variant & refval = variant(),  const variant & incr = variant(),  const variant & xform = variant(),  const variant & poles = variant());

         bool setepoch( const record& value);

         bool setincrement( const variant & value = variant(),  const string& type=string(""));

         bool setlineartransform( const string& type=string(""),  const variant & value = variant());

         bool setnames( const  std::vector<std::string>& value,  const string& type=string(""));

         bool setobserver( const string& value);

         bool setprojection( const string& type,  const std::vector<double>& parameters=initialize_vector(1, (double)-1));

         bool setreferencecode( const string& value,  const string& type=string("direction"),  bool  adjust=true);

         bool setreferencelocation( const  std::vector<int>& pixel=initialize_vector(1, (int)-1),  const variant & world = variant(),  const  std::vector<bool>&  mask=initialize_vector(1, (bool)false));

         bool setreferencepixel( const std::vector<double>& value,  const string& type=string(""));

         bool setreferencevalue( const variant & value,  const string& type=string(""));

         bool setrestfrequency( const variant & value,  int  which=0,  bool  append=false);

         bool setspectral( const string& refcode=string(""),  const variant & restfreq = variant(),  const variant & frequencies = variant(),  const string& doppler=string(""),  const variant & velocities = variant());

         bool setstokes( const  std::vector<std::string>& stokes);

         bool settabular( const std::vector<double>& pixel=initialize_vector(1, (double)-1),  const std::vector<double>& world=initialize_vector(1, (double)-1),  int  which=0);

         bool settelescope( const string& value);

         bool setunits( const  std::vector<std::string>& value,  const string& type=string(""),  bool  overwrite=false,  int  which=-10);

         std::vector<std::string> stokes();

         std::vector<std::string> summary( const string& doppler=string("RADIO"),  bool  list=true);

         string telescope();

         record* toabs( const variant & value,  int  isworld=-1);

         record* toabsmany( const variant & value,  int  isworld=-1);

         record* topixel( const variant & value);

         record* topixelmany( const variant & value);

         record* torecord();

         record* subimage( const variant & originshft,  const  std::vector<int>& newshape=std::vector<int>() );

         record* torel( const variant & value,  int  isworld=-1);

         record* torelmany( const variant & value,  int  isworld=-1);

         record* toworld( const variant & value = variant(),  const string& format=string("n"));

         record* toworldmany( const variant & value);

         string type();

         std::vector<std::string> units( const string& type=string(""));

         std::vector<double>velocitytofrequency( const std::vector<double>& value,  const string& frequnit=string(""),  const string& doppler=string("radio"),  const string& velunit=string("km/s"));

         string parentname();

         bool setparentname( const string& imagename);


      private :
#include <tools/images/coordsys_private.h>
   }; 

 
}
#endif
