  
#ifndef _ms_cmpt_h_
#define _ms_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <table_cmpt.h>
#include <tools/ms/ms_forward.h>

using namespace std;

namespace casac {
           


   class  ms  {
       public :


         ~ms();
         ms();

         bool fromfits( const string& msfile=string(""),  const string& fitsfile=string(""),  bool  nomodify=true,  bool  lock=false,  int  obstype=0,  const string& host=string(""),  bool  forcenewserver=false, const  string& antnamescheme=string("old"));

         bool fromfitsidi( const string& msfile=string(""),  const string& fitsfile=string(""),  bool  nomodify=true,  bool  lock=false,  int  obstype=0);

         bool listfits(const  string& fitsfile);

         bool createmultims(const  string& outputTableName,  const std::vector<std::string>& tables,  const std::vector<std::string>& subtables,  bool  nomodify=true,  bool  lock=false,  bool  copysubtables=false,  const std::vector<std::string>& omitsubtables=std::vector<string>(1, ""));

         bool ismultims();

         std::vector<std::string> getreferencedtables();

         variant* getfielddirmeas( const string& dircolname=string("PHASE_DIR"),  int  fieldid=0, 
			  double time=0, const  string& format=string("measure"));

         int nrow( bool  selected=false);

         int nrow2( bool  selected=false);

         bool iswritable();

         bool open( const string& thems=string(""),  bool  nomodify=true,  bool  lock=false,  bool  check=false);

         bool reset();

         bool close();

         string name();

         bool statwt2(const  string& combine=string(""),  const variant & timebin= variant(),  bool  slidetimebin=false,  const variant & chanbin= variant(),  int  minsamp=2, const  string& statalg=string("classic"), 
			  double fence=-1, const  string& center=string("mean"),  bool  lside=true, 
			  double zscore=-1,  int  maxiter=-1, const  string& excludechans=string(""),  const std::vector<double>& wtrange=std::vector<double>() ,  bool  preview=false, const  string& datacolumn=string("corrected"));

         bool tofits( const string& fitsfile=string(""),  const string& column=string("corrected"),  const variant & field = variant(),  const variant & spw = variant(),  const variant & baseline = variant(),  const string& time=string(""),  const variant & scan= variant(),  const variant & uvrange= variant(), const  string& taql=string(""),  bool  writesyscal=false,  bool  multisource=false,  bool  combinespw=false,  bool  writestation=false,  bool  padwithflags=false,  bool  overwrite=false);

         record* summary( bool  verbose=false,  const string& listfile=string(""),  bool  listunfl=false, 
			  double cachesize=50,  bool  overwrite=false,  bool  wantreturn=true);

         record* getscansummary();

         record* getspectralwindowinfo();

         bool listhistory();

         bool writehistory( const string& message=string(""),  const string& parms=string(""),  const string& origin=string("MSHistoryHandler::addMessage()"),  const string& msname=string(""),  const string& app=string("ms"));

         record* statisticsold(const  string& column=string(""), const  string& complex_value=string(""),  bool  useflags=true, const  string& spw=string(""), const  string& field=string(""), const  string& baseline=string(""), const  string& uvrange=string(""), const  string& time=string(""), const  string& correlation=string(""), const  string& scan=string(""), const  string& array=string(""), const  string& obs=string(""));

         record* statistics(const  string& column=string(""), const  string& complex_value=string(""),  bool  useflags=true,  bool  useweights=false, const  string& spw=string(""), const  string& field=string(""), const  string& baseline=string(""), const  string& uvrange=string(""), const  string& time=string(""), const  string& correlation=string(""), const  string& scan=string(""), const  string& intent=string(""), const  string& array=string(""), const  string& obs=string(""), const  string& reportingaxes=string(""),  bool  timeaverage=false, const  string& timebin=string("0s"), const  string& timespan=string(""), 
			  double maxuvwdistance=0.0);

         record* range( const  std::vector<std::string>& items=std::vector<string>(1, ""),  bool  useflags=true,  int  blocksize=10);

         record* range2( const  std::vector<std::string>& items=std::vector<string>(1, ""),  bool  useflags=true,  int  blocksize=10);

         bool lister( const string& options=string(""),  const string& datacolumn=string("data"),  const string& field=string(""),  const string& spw=string(""),  const string& antenna=string(""),  const string& timerange=string(""),  const string& correlation=string(""),  const string& scan=string(""),  const string& feed=string(""),  const string& array=string(""),  const string& observation=string(""),  const string& uvrange=string(""),  const string& average=string(""),  bool  showflags=false,  const string& msselect=string(""),  int  pagerows=50,  const string& listfile=string(""));

         
                                      casac::msmetadata* metadata( float  cachesize=50);

         bool selectinit( int  datadescid=0,  bool  reset=false);

         bool selectinit2( int  datadescid=0,  bool  reset=false);

         bool msselect( const record& items=initialize_record("") ,  bool  onlyparse=false);

         record* msselectedindices();

         bool select( const record& items=initialize_record("") );

         bool select2( const record& items=initialize_record("") );

         bool selecttaql( const string& msselect=string(""));

         bool selecttaql2( const string& msselect=string(""));

         bool selectchannel( int  nchan=1,  int  start=0,  int  width=1,  int  inc=1);

         bool selectchannel2( int  nchan=1,  int  start=0,  int  width=1,  int  inc=1);

         bool selectpolarization( const std::vector<std::string>& wantedpol=std::vector<string>(1, ""));

         bool selectpolarization2( const std::vector<std::string>& wantedpol=std::vector<string>(1, ""));

         bool regridspw( const string& outframe=string("LSRK"),  const string& mode=string("chan"), 
			  double restfreq=-3E30,  const string& interpolation=string("LINEAR"), 
			  double start=-3E30, 
			  double center=-3E30, 
			  double bandwidth=-1., 
			  double chanwidth=-1.,  bool  hanning=true);

         bool cvel( const string& mode=string("channel"),  int  nchan=-1,  const variant & start = variant(),  const variant & width = variant(),  const string& interp=string("linear"),  const variant & phasec = variant(),  const variant & restfreq = variant(),  const string& outframe=string(""),  const string& veltype=string("radio"),  bool  hanning=true);

         std::vector<double>cvelfreqs( const  std::vector<int>& spwids=initialize_vector(1, (int)0),  const  std::vector<int>& fieldids=initialize_vector(1, (int)0),  const string& obstime=string(""),  const string& mode=string("channel"),  int  nchan=-1,  const variant & start = variant(),  const variant & width = variant(),  const variant & phasec = variant(),  const variant & restfreq = variant(),  const string& outframe=string(""),  const string& veltype=string("radio"),  bool  verbose=true);

         record* getdata( const  std::vector<std::string>& items=std::vector<string>(1, ""),  bool  ifraxis=false,  int  ifraxisgap=0,  int  increment=1,  bool  average=false);

         record* getdata2( const  std::vector<std::string>& items=std::vector<string>(1, ""),  bool  ifraxis=false,  int  ifraxisgap=0,  int  increment=1,  bool  average=false);

         bool putdata( const record& items=initialize_record("") );

         bool putdata2( const record& items=initialize_record("") );

         bool concatenate( const string& msfile=string(""),  const variant & freqtol = variant(),  const variant & dirtol = variant(),  float  weightscale=1.,  int  handling=0,  const string& destmsfile=string(""),  bool  respectname=false);

         bool testconcatenate( const string& msfile=string(""),  const variant & freqtol = variant(),  const variant & dirtol = variant(),  bool  respectname=false);

         bool virtconcatenate( const string& msfile=string(""),  const string& auxfilename=string(""),  const variant & freqtol = variant(),  const variant & dirtol = variant(),  float  weightscale=1.,  bool  respectname=true);

         bool timesort( const string& newmsname=string(""));

         bool sort( const string& newmsname=string(""),  const  std::vector<std::string>& columns=std::vector<string>(1, ""));

         bool contsub( const string& outputms=string(""),  const variant & fitspw = variant(),  int  fitorder=1,  const string& combine=string(""),  const variant & spw = variant(),  const variant & unionspw = variant(),  const variant & field = variant(),  const variant & scan = variant(),  const string& intent=string(""),  const string& correlation=string(""),  const string& obs=string(""),  const string& whichcol=string("CORRECTED_DATA"));

         bool statwt( bool  dorms=false,  bool  byantenna=true,  bool  sepacs=true,  const variant & fitspw = variant(),  const variant & fitcorr = variant(),  const string& combine=string(""),  const variant & timebin = variant(),  int  minsamp=3,  const variant & field = variant(),  const variant & spw = variant(),  const variant & antenna= variant(), const  string& timerange=string(""),  const variant & scan = variant(),  const string& intent=string(""),  const variant & array = variant(),  const string& correlation=string(""),  const string& obs=string(""),  const string& datacolumn=string("corrected_data"));

         bool split( const string& outputms=string(""),  const variant & field = variant(),  const variant & spw = variant(),  const  std::vector<int>& step=initialize_vector(1, (int)1),  const variant & baseline = variant(),  const variant & timebin = variant(),  const string& time=string(""),  const variant & scan = variant(),  const variant & uvrange = variant(), const  string& taql=string(""),  const string& whichcol=string("DATA"),  const variant & tileshape = variant(),  const variant & subarray = variant(),  const string& combine=string(""),  const string& correlation=string(""),  const string& intent=string(""),  const string& obs=string(""));

         bool partition( const string& outputms=string(""),  const variant & field = variant(),  const variant & spw = variant(),  const variant & baseline = variant(),  const variant & timebin = variant(),  const string& time=string(""),  const variant & scan = variant(),  const variant & uvrange = variant(), const  string& taql=string(""),  const string& whichcol=string("DATA"),  const variant & tileshape = variant(),  const variant & subarray = variant(),  const string& combine=string(""),  const string& intent=string(""),  const string& obs=string(""));

         bool iterinit( const  std::vector<std::string>& columns=std::vector<string>(1, ""), 
			  double interval=0.0,  int  maxrows=0,  bool  adddefaultsortcolumns=true);

         bool iterinit2( const  std::vector<std::string>& columns=std::vector<string>(1, ""), 
			  double interval=0.0,  int  maxrows=0,  bool  adddefaultsortcolumns=true);

         bool iterorigin();

         bool iterorigin2();

         bool iternext();

         bool iternext2();

         bool iterend();

         bool iterend2();

         string asdmref(const  string& abspath=string(""));

         bool continuumsub( const variant & field = variant(),  const variant & fitspw = variant(),  const variant & spw = variant(),  const variant & solint = variant(),  int  fitorder=0,  const string& mode=string("subtract"));

         bool continuumsub2( const variant & field = variant(),  const variant & fitspw = variant(),  const variant & spw = variant(),  const variant & solint = variant(),  int  fitorder=0,  const string& mode=string("subtract"));

         bool done();

         record* msseltoindex(const  string& vis=string(""),  const variant & spw= variant(),  const variant & field= variant(),  const variant & baseline= variant(),  const variant & time= variant(),  const variant & scan= variant(),  const variant & uvrange= variant(),  const variant & observation= variant(),  const variant & polarization= variant(), const  string& taql=string(""));

         bool hanningsmooth( const string& datacolumn=string("corrected"));

         bool uvsub( bool  reverse=false);

         bool addephemeris( int  id=-1,  const string& ephemerisname=string(""),  const string& comment=string(""),  const variant & field = variant());

         record* ngetdata( const  std::vector<std::string>& items=std::vector<string>(1, ""),  bool  ifraxis=false,  int  ifraxisgap=0,  int  increment=1,  bool  average=false);

         bool niterinit( const  std::vector<std::string>& columns=std::vector<string>(1, ""), 
			  double interval=0.0,  int  maxrows=0,  bool  adddefaultsortcolumns=true);

         bool niterorigin();

         bool niternext();

         bool niterend();


      private :
#include <tools/ms/ms_private.h>
   }; 

 
}
#endif
