  
#ifndef _plotms_cmpt_h_
#define _plotms_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/plotms/plotms_forward.h>

using namespace std;

namespace casac {
           


   class  plotms  {
       public :


         ~plotms();
         plotms();

         void setLogFilename(const  string& logFilename=string("casapy.log"));

         string getLogFilename();

         void setLogFilter(const  string& priority);

         string getLogFilter();

         void setPlotmsPid( int  pid);

         void setClearSelectionOnAxesChange( bool  clearSelection);

         bool getClearSelectionOnAxesChange();

         void setCachedImageSize( int  width,  int  height);

         void setCachedImageSizeToScreenResolution();

         int getCachedImageWidth();

         int getCachedImageHeight();

         void setGridSize( int  rowCount=1,  int  colCount=1);

         void setPlotMSFilename(const  string& msFilename,  bool  updateImmediately=true,  int  plotIndex=0);

         string getPlotMSFilename( int  plotIndex=0);

         void setcallib(const  string& callib,  bool  updateImmediately=true,  int  plotIndex=0);

         void setPlotMSSelection(const  string& field=string(""), const  string& spw=string(""), const  string& timerange=string(""), const  string& uvrange=string(""), const  string& antenna=string(""), const  string& scan=string(""), const  string& corr=string(""), const  string& array=string(""), const  string& observation=string(""), const  string& intent=string(""), const  string& feed=string(""), const  string& msselect=string(""),  bool  updateImmediately=true,  int  plotIndex=0);

         void setPlotMSSelectionRec( const record& selection,  bool  updateImmediately=true,  int  plotIndex=0);

         record* getPlotMSSelection( int  plotIndex=0);

         void setPlotMSAveraging(const  string& channel=string(""), const  string& time=string(""),  bool  scan=false,  bool  field=false,  bool  baseline=false,  bool  antenna=false,  bool  spw=false,  bool  scalar=false,  bool  updateImmediately=true,  int  plotIndex=0);

         void setPlotMSAveragingRec( const record& averaging,  bool  updateImmediately=true,  int  plotIndex=0);

         record* getPlotMSAveraging( int  plotIndex=0);

         void setPlotMSTransformations(const  string& freqframe=string(""), const  string& veldef=string(""),  const variant & restfreq= variant(), 
			  double xshift=0.0, 
			  double yshift=0.0,  bool  updateImmediately=true,  int  plotIndex=0);

         void setPlotMSCalibration( bool  use=false, const  string& callibrary=string(""),  bool  updateImmediately=true,  int  plotIndex=0);

         void setPlotMSIterate(const  string& iteraxis=string(""),  int  rowIndex=0,  int  colIndex=0,  bool  xselfscale=false,  bool  yselfscale=false,  bool  commonAxisX=false,  bool  commonAxisY=false,  bool  updateImmediately=true,  int  plotIndex=0);

         void setPlotMSTransformationsRec( const record& transformations,  bool  updateImmediately=true,  int  plotIndex=0);

         record* getPlotMSTransformations( int  plotIndex=0);

         record* getPlotMSCalibration( int  plotIndex=0);

         void setExportRange(const  string& range=string(""));

         void setPlotMSPageHeaderItems(const  string& pageHeaderItems,  bool  updateImmediately=true,  int  plotIndex=0);

         void setTitle(const  string& text=string(""),  bool  updateImmediately=true,  int  plotIndex=0);

         void setTitleFont( int  size=0,  bool  updateImmediately=true,  int  plotIndex=0);

         void setXAxisLabel(const  string& text=string(""),  bool  updateImmediately=true,  int  plotIndex=0);

         void setXAxisFont( int  size=0,  bool  updateImmediately=true,  int  plotIndex=0);

         void setYAxisLabel(const  string& text=string(""),  bool  updateImmediately=true,  int  plotIndex=0);

         void setYAxisFont( int  size=0,  bool  updateImmediately=true,  int  plotIndex=0);

         string getTitle( int  plotIndex=0);

         string getXAxisLabel( int  plotIndex=0);

         string getYAxisLabel( int  plotIndex=0);

         void setPlotAxes(const  string& xAxis=string(""), const  string& yAxis=string(""), const  string& xDataColumn=string(""), const  string& yDataColumn=string(""), const  string& yAxisLocation=string(""),  bool  updateImmediately=true,  int  plotIndex=0,  int  dataIndex=0);

         string getPlotXAxis( int  plotIndex=0);

         string getPlotXDataColumn( int  plotIndex=0);

         string getPlotYAxis( int  plotIndex=0);

         string getPlotYDataColumn( int  plotIndex=0);

         record* getPlotParams( int  plotIndex=0);

         void setXRange( bool  xautorange=true, 
			  double xmin=0, 
			  double xmax=0,  bool  updateImmediately=true,  int  plotIndex=0);

         void setYRange( bool  yautorange=true, 
			  double ymin=0, 
			  double ymax=0,  bool  updateImmediately=true,  int  plotIndex=0);

         void setColorizeFlag( bool  colorize,  bool  updateImmediately=true,  int  plotIndex=0);

         bool getColorizeFlag( int  plotIndex=0);

         void setColorAxis(const  string& coloraxis,  bool  updateImmediately=true,  int  plotIndex=0);

         void setSymbol(const  string& shape=string("autoscaling"),  int  size=2, const  string& color=string("0000ff"), const  string& fill=string("fill"),  bool  outline=false,  bool  updateImmediately=true,  int  plotIndex=0,  int  dataIndex=0);

         void setFlaggedSymbol(const  string& shape=string("nosymbol"),  int  size=2, const  string& color=string("ff0000"), const  string& fill=string("fill"),  bool  outline=false,  bool  updateImmediately=true,  int  plotIndex=0,  int  dataIndex=0);

         string getColorAxis( int  plotIndex=0);

         void setFlagExtension( bool  extend=false, const  string& correlation=string(""),  bool  channel=false,  bool  spw=false, const  string& antenna=string(""),  bool  time=false,  bool  scans=false,  bool  field=false,  const record& alternateSelection=initialize_record("") );

         void setFlagExtensionRec( const record& flagExtension);

         record* getFlagExtension();

         record* locateInfo();

         bool getGridMajorShown( int  plotIndex=0);

         bool getGridMinorShown( int  plotIndex=0);

         int getGridMajorWidth( int  plotIndex=1);

         int getGridMinorWidth( int  plotIndex=1);

         void setGridParams( bool  showmajorgrid=false,  int  majorwidth=1, const  string& majorstyle=string("solid"), const  string& majorcolor=string("B0B0B0"),  bool  showminorgrid=false,  int  minorwidth=1, const  string& minorstyle=string("solid"), const  string& minorcolor=string("C0CCE0"),  bool  updateImmediately=true,  int  plotIndex=0);

         void setLegend( bool  showLegend=false, const  string& legendPosition=string("upperright"),  bool  updateImmediately=true,  int  plotIndex=0);

         bool update();

         bool save(const  string& plotfile, const  string& format,  bool  highres=false,  int  dpi=-1,  int  width=-1,  int  height=-1);

         bool isDrawing();

         bool isClosed();

         void setShowGui( bool  show);

         void clearPlots();

         void killApp();


      private :
#include <tools/plotms/plotms_private.h>
   }; 

 
}
#endif
