  
#ifndef _table_cmpt_h_
#define _table_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/tables/table_forward.h>

using namespace std;

namespace casac {
           


   class  table  {
       public :


         ~table();
         table();

         
                                      casac::table* fromfits(const  string& tablename, const  string& fitsfile,  int  whichhdu=1, const  string& storage=string("standard"), const  string& convention=string("none"),  bool  nomodify=true,  bool  ack=true);

         bool fromascii(const  string& tablename, const  string& asciifile, const  string& headerfile=string(""),  bool  autoheader=false,  const std::vector<int>& autoshape=initialize_vector(1, (int)-1), const  string& sep=string(""), const  string& commentmarker=string(""),  int  firstline=0,  int  lastline=-1,  bool  nomodify=true,  const std::vector<std::string>& columnnames=std::vector<string>(1, ""),  const std::vector<std::string>& datatypes=std::vector<string>(1, ""));

         bool open(const  string& tablename=string(""),  const record& lockoptions=initialize_record("") ,  bool  nomodify=true);

         bool create(const  string& tablename=string(""),  const record& tabledesc=initialize_record("") ,  const record& lockoptions=initialize_record("default"), const  string& endianformat=string(""), const  string& memtype=string(""),  int  nrow=0,  const record& dminfo=initialize_record("") );

         bool flush();

         bool fromASDM(const  string& tablename, const  string& xmlfile);

         bool resync();

         bool close();

         
                                      casac::table* copy(const  string& newtablename,  bool  deep=false,  bool  valuecopy=false,  const record& dminfo=initialize_record("") , const  string& endian=string("aipsrc"),  bool  memorytable=false,  bool  returnobject=false,  bool  norows=false);

         bool copyrows(const  string& outtable,  int  startrowin=0,  int  startrowout=-1,  int  nrow=-1);

         bool done();

         bool iswritable();

         string endianformat();

         bool lock( bool  write=true,  int  nattempts=0);

         bool unlock();

         bool datachanged();

         bool haslock( bool  write=true);

         record* lockoptions();

         bool ismultiused( bool  checksubtables=false);

         bool browse();

         string name();

         bool createmultitable(const  string& outputTableName,  const std::vector<std::string>& tables, const  string& subdirname);

         bool toasciifmt(const  string& asciifile, const  string& headerfile=string(""),  const std::vector<std::string>& columns=std::vector<string>(1, ""), const  string& sep=string(""));

         
                                      casac::table* taql(const  string& taqlcommand=string("TaQL expression"));

         
                                      casac::table* query(const  string& query=string("String"), const  string& name=string(""), const  string& sortlist=string(""), const  string& columns=string(""), const  string& style=string(""));

         variant* calc(const  string& expr, const  string& prefix=string("using style base0, endincl, fortranorder"),  bool  showtaql=false);

         
                                      casac::table* selectrows( const std::vector<int>& rownrs, const  string& name=string(""));

         record* info();

         bool putinfo( const record& value);

         bool addreadmeline(const  string& value);

         bool summary( bool  recurse=false);

         std::vector<std::string> colnames();

         std::vector<int> rownumbers( const record& tab=initialize_record("") ,  int  nbytes=0);

         bool setmaxcachesize(const  string& columnname,  int  nbytes);

         bool isscalarcol(const  string& columnname);

         bool isvarcol(const  string& columnname);

         string coldatatype(const  string& columnname);

         string colarraytype(const  string& columnname);

         int ncols();

         int nrows();

         bool addrows( int  nrow=1);

         bool removerows( const std::vector<int>& rownrs);

         bool addcols( const record& desc,  const record& dminfo=initialize_record("") );

         bool renamecol(const  string& oldname, const  string& newname);

         bool removecols( const std::vector<std::string>& columnames);

         bool iscelldefined(const  string& columnname,  int  rownr=0);

         variant* getcell(const  string& columnname,  int  rownr=0);

         variant* getcellslice(const  string& columnname,  int  rownr,  const std::vector<int>& blc,  const std::vector<int>& trc,  const std::vector<int>& incr=initialize_vector(1, (int)1));

         variant* getcol(const  string& columnname,  int  startrow=0,  int  nrow=-1,  int  rowincr=1);

         record* getvarcol(const  string& columnname,  int  startrow=0,  int  nrow=-1,  int  rowincr=1);

         variant* getcolslice(const  string& columnname,  const std::vector<int>& blc,  const std::vector<int>& trc,  const std::vector<int>& incr,  int  startrow=0,  int  nrow=-1,  int  rowincr=1);

         bool putcell(const  string& columnname,  const std::vector<int>& rownr,  const variant & thevalue);

         bool putcellslice(const  string& columnname,  int  rownr,  const variant & value,  const std::vector<int>& blc,  const std::vector<int>& trc,  const std::vector<int>& incr=initialize_vector(1, (int)1));

         bool putcol(const  string& columnname,  const variant & value,  int  startrow=0,  int  nrow=-1,  int  rowincr=1);

         bool putvarcol(const  string& columnname,  const record& value,  int  startrow=0,  int  nrow=-1,  int  rowincr=1);

         bool putcolslice(const  string& columnname,  const variant & value,  const std::vector<int>& blc,  const std::vector<int>& trc,  const std::vector<int>& incr=initialize_vector(1, (int)1),  int  startrow=0,  int  nrow=-1,  int  rowincr=1);

         std::vector<std::string> getcolshapestring(const  string& columnname,  int  startrow=0,  int  nrow=-1,  int  rowincr=1);

         variant* getkeyword( const variant & keyword);

         record* getkeywords();

         variant* getcolkeyword(const  string& columnname,  const variant & keyword);

         variant* getcolkeywords(const  string& columnname);

         bool putkeyword( const variant & keyword,  const variant & value,  bool  makesubrecord=false);

         bool putkeywords( const record& value);

         bool putcolkeyword(const  string& columnname,  const variant & keyword,  const variant & value);

         bool putcolkeywords(const  string& columnname,  const record& value);

         bool removekeyword( const variant & keyword);

         bool removecolkeyword(const  string& columnname,  const variant & keyword);

         record* getdminfo();

         std::vector<std::string> keywordnames();

         std::vector<std::string> fieldnames(const  string& keyword=string(""));

         std::vector<std::string> colkeywordnames(const  string& columnname);

         std::vector<std::string> colfieldnames(const  string& columnname, const  string& keyword=string(""));

         record* getdesc( bool  actual=true);

         record* getcoldesc(const  string& columnname);

         bool ok();

         bool clearlocks();

         bool listlocks();

         record* statistics(const  string& column, const  string& complex_value=string(""),  bool  useflags=true);

         std::vector<std::string> showcache( bool  verbose=true);

         bool testincrstman(const  string& column);


      private :
#include <tools/tables/table_private.h>
   }; 

 
}
#endif
